==> vdr-plugin-webvideo_0.2.1.orig.tar.gz: webvideo-0.2.1/src/vdr-plugin/request.c <==
/*
 * request.c: Web video plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <vdr/tools.h>
#include <vdr/i18n.h>
#include "request.h"
#include "common.h"
#include "mimetypes.h"

// --- cDownloadProgress ---------------------------------------------------

cDownloadProgress::cDownloadProgress(const char *title, 
                                     long downloadedbytes, 
                                     long totalbytes,
                                     long downloadid) {
  name = title ? strdup(title) : NULL;
  downloaded = downloadedbytes;
  total = totalbytes;
  id = downloadid;
}

cDownloadProgress::~cDownloadProgress() {
  if (name)
    free(name);
}

const char *cDownloadProgress::GetTitle() {
  if (name)
    return name;
  else
    return "???";
}

float cDownloadProgress::GetPercentage() {
  if ((downloaded < 0) || (total < 0))
    return 0.0;
  else
    return float(downloaded)/total;
}

// --- cMenuRequest --------------------------------------------------------

cMenuRequest::cMenuRequest(int ID, const char *wvtreference)
: reqID(ID), aborted(false), finished(false), status(0), webvi(-1), handle(-1)
{
  wvtref = strdup(wvtreference);
}

cMenuRequest::~cMenuRequest() {
  if (handle != -1) {
    if (!finished)
      Abort();
    webvi_delete_handle(webvi, handle);
  }
}

ssize_t cMenuRequest::WriteCallback(const char *ptr, size_t len, void *request) {
  cMenuRequest *instance = (cMenuRequest *)request;
  if (instance)
    return instance->WriteData(ptr, len);
  else
    return len;
}

ssize_t cMenuRequest::WriteData(const char *ptr, size_t len) {
  return inBuffer.Put(ptr, len);
}

WebviHandle cMenuRequest::PrepareHandle() {
  if (handle == -1) {
    handle = webvi_new_request(webvi, wvtref, WEBVIREQ_MENU);

    if (handle != -1) {
      webvi_set_opt(webvi, handle, WEBVIOPT_WRITEFUNC, WriteCallback);
      webvi_set_opt(webvi, handle, WEBVIOPT_WRITEDATA, this);
    }
  }

  return handle;
}

bool cMenuRequest::Start(WebviCtx webvictx) {
  webvi = webvictx;

  if ((PrepareHandle() != -1) && (webvi_start_handle(webvi, handle) == WEBVIERR_OK)) {
    finished = false;
    return true;
  } else 
    return false;
}

void cMenuRequest::RequestDone(int errorcode, cString pharse) {
  finished = true;
  status = errorcode;
  statusPharse = pharse;
}

void cMenuRequest::Abort() {
  if (finished || handle == -1)
    return;

  aborted = true;
  webvi_stop_handle(webvi, handle);
};

bool cMenuRequest::Success() {
  return status == 0;
}

cString cMenuRequest::GetStatusPharse() {
  return statusPharse;
}

cString cMenuRequest::GetResponse() {
  size_t len = inBuffer.Length();
  const char *src = inBuffer.Get();
  char *buf = (char *)malloc((len+1)*sizeof(char));
  strncpy(buf, src, len);
  buf[len] = '\0';
  return cString(buf, true);
}

// --- cFileDownloadRequest ------------------------------------------------

cFileDownloadRequest::cFileDownloadRequest(int ID, const char *streamref, 
                                           const char *destdir)
:  cMenuRequest(ID, streamref), title(NULL), bytesDownloaded(0),
   contentLength(-1), destfile(NULL)
{
  this->destdir = strdup(destdir);
}

cFileDownloadRequest::~cFileDownloadRequest() {
  if (destfile) {
    destfile->Close();
    delete destfile;
  }
  if (destdir)
    free(destdir);
  if (title)
    free(title);
}

WebviHandle cFileDownloadRequest::PrepareHandle() {
  if (handle == -1) {
    handle = webvi_new_request(webvi, wvtref, WEBVIREQ_FILE);

    if (handle != -1) {
      webvi_set_opt(webvi, handle, WEBVIOPT_WRITEFUNC, WriteCallback);
      webvi_set_opt(webvi, handle, WEBVIOPT_WRITEDATA, this);
    }
  }

  return handle;
}

ssize_t cFileDownloadRequest::WriteData(const char *ptr, size_t len) {
  if (!destfile) {
    if (!OpenDestFile())
      return -1;
  }

  bytesDownloaded += len;
  return destfile->Write(ptr, len);
}

bool cFileDownloadRequest::OpenDestFile() {
  char *contentType;
  char *url;
  char *ext;
  cString destfilename;
  int fd, i;

  if (handle == -1)
    return false;

  if (destfile)
    delete destfile;

  destfile = new cUnbufferedFile;

  webvi_get_info(webvi, handle, WEBVIINFO_URL, &url);
  webvi_get_info(webvi, handle, WEBVIINFO_STREAM_TITLE, &title);
  webvi_get_info(webvi, handle, WEBVIINFO_CONTENT_TYPE, &contentType);
  webvi_get_info(webvi, handle, WEBVIINFO_CONTENT_LENGTH, &contentLength);

  debug("url: %s", url);
  debug("title: %s", title);
  debug("content type: %s", contentType);
  debug("content length: %ld", contentLength);

  if (!contentType || !url) {
    if(contentType)
      free(contentType);
    if (url)
      free(url);
    return false;
  }

  ext = GetExtension(contentType, url);

  free(url);
  free(contentType);

  char *basename = strdup(title ? title : "???");
  basename = safeFilename(basename);

  i = 1;
  destfilename = cString::sprintf("%s/%s%s", destdir, basename, ext);
  while (true) {
    debug("trying to open %s", (const char *)destfilename);

    fd = destfile->Open(destfilename, O_WRONLY | O_CREAT | O_EXCL, DEFFILEMODE);

    if (fd == -1 && errno == EEXIST)
      destfilename = cString::sprintf("%s/%s-%d%s", destdir, basename, i++, ext);
    else
      break;
  };

  free(basename);
  free(ext);

  if (fd < 0) {
    error("Failed to open file %s: %m", (const char *)destfilename);
    delete destfile;
    destfile = NULL;
    return false;
  }

  info("Saving to %s", (const char *)destfilename);

  return true;
}

char *cFileDownloadRequest::GetExtension(const char *contentType, const char *url) {
  // Get extension from Content-Type
  char *ext = NULL;
  char *ext2 = MimeTypes->ExtensionFromMimeType(contentType);

  // Workaround for buggy servers: If the server claims that the mime
  // type is text/plain, ignore the server and fall back to extracting
  // the extension from the URL. This function should be called only
  // for video, audio or ASX files and therefore text/plain is clearly
  // incorrect.
  if (ext2 && contentType && !strcasecmp(contentType, "text/plain")) {
    debug("Ignoring content type text/plain, getting extension from url.");
    free(ext2);
    ext2 = NULL;
  }

  if (ext2) {
    // Append dot in the start of the extension
    ext = (char *)malloc(strlen(ext2)+2);
    ext[0] = '.';
    ext[1] = '\0';
    strcat(ext, ext2);
    free(ext2);
    return ext;
  }

  // Get extension from URL
  ext = extensionFromUrl(url);
  if (ext)
    return ext;

  // No extension!
  return strdup("");
}

void cFileDownloadRequest::RequestDone(int errorcode, cString pharse) {
  cMenuRequest::RequestDone(errorcode, pharse);
  if (destfile)
    destfile->Close();
}

cDownloadProgress *cFileDownloadRequest::GetProgress(long downloadid) {
  const char *name;
  if (title)
    name = title;
  else
    name = "???";

  cString dltitle;
  if (IsAborted()) {
    dltitle = cString::sprintf("[%s] %s", tr("Aborted"), name);
  } else {
    dltitle = cString(name);
  }

  return new cDownloadProgress(dltitle, bytesDownloaded, contentLength, 
                               downloadid);
}

// --- cStreamUrlRequest ---------------------------------------------------

WebviHandle cStreamUrlRequest::PrepareHandle() {
  if (handle == -1) {
    handle = webvi_new_request(webvi, wvtref, WEBVIREQ_STREAMURL);

    if (handle != -1) {
      webvi_set_opt(webvi, handle, WEBVIOPT_WRITEFUNC, WriteCallback);
      webvi_set_opt(webvi, handle, WEBVIOPT_WRITEDATA, this);
    }
  }

  return handle;
}

// --- cRequestVector ------------------------------------------------------

cMenuRequest *cRequestVector::FindByHandle(WebviHandle handle) {
  for (int i=0; i<Size(); i++)
    if (At(i)->GetHandle() == handle)
      return At(i);

  return NULL;
}

==> vdr-plugin-webvideo_0.2.2.orig.tar.gz: webvideo-0.2.2/src/vdr-plugin/request.c <==
/*
 * request.c: Web video plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <vdr/tools.h>
#include <vdr/i18n.h>
#include "request.h"
#include "common.h"
#include "mimetypes.h"

// --- cDownloadProgress ---------------------------------------------------

cDownloadProgress::cDownloadProgress(const char *title, 
                                     long downloadedbytes, 
                                     long totalbytes,
                                     tProgressID downloadid) {
  name = title ? strdup(title) : NULL;
  downloaded = downloadedbytes;
  total = totalbytes;
  id = downloadid;
}

cDownloadProgress::~cDownloadProgress() {
  if (name)
    free(name);
}

const char *cDownloadProgress::GetTitle() {
  if (name)
    return name;
  else
    return "???";
}

float cDownloadProgress::GetPercentage() {
  if ((downloaded < 0) || (total < 0))
    return -1.0;
  else
    return float(downloaded)/total;
}

// --- cMenuRequest --------------------------------------------------------

cMenuRequest::cMenuRequest(int ID, const char *wvtreference)
: reqID(ID), aborted(false), finished(false), status(0), webvi(-1), handle(-1)
{
  wvtref = strdup(wvtreference);
}

cMenuRequest::~cMenuRequest() {
  if (handle != -1) {
    if (!finished)
      Abort();
    webvi_delete_handle(webvi, handle);
  }
}

ssize_t cMenuRequest::WriteCallback(const char *ptr, size_t len, void *request) {
  cMenuRequest *instance = (cMenuRequest *)request;
  if (instance)
    return instance->WriteData(ptr, len);
  else
    return len;
}

ssize_t cMenuRequest::WriteData(const char *ptr, size_t len) {
  return inBuffer.Put(ptr, len);
}

WebviHandle cMenuRequest::PrepareHandle() {
  if (handle == -1) {
    handle = webvi_new_request(webvi, wvtref, WEBVIREQ_MENU);

    if (handle != -1) {
      webvi_set_opt(webvi, handle, WEBVIOPT_WRITEFUNC, WriteCallback);
      webvi_set_opt(webvi, handle, WEBVIOPT_WRITEDATA, this);
    }
  }

  return handle;
}

bool cMenuRequest::Start(WebviCtx webvictx) {
  webvi = webvictx;

  if ((PrepareHandle() != -1) && (webvi_start_handle(webvi, handle) == WEBVIERR_OK)) {
    finished = false;
    return true;
  } else 
    return false;
}

void cMenuRequest::RequestDone(int errorcode, cString pharse) {
  finished = true;
  status = errorcode;
  statusPharse = pharse;
}

void cMenuRequest::Abort() {
  if (finished || handle == -1)
    return;

  aborted = true;
  webvi_stop_handle(webvi, handle);
};

bool cMenuRequest::Success() {
  return status == 0;
}

cString cMenuRequest::GetStatusPharse() {
  return statusPharse;
}

cString cMenuRequest::GetResponse() {
  size_t len = inBuffer.Length();
  const char *src = inBuffer.Get();
  char *buf = (char *)malloc((len+1)*sizeof(char));
  strncpy(buf, src, len);
  buf[len] = '\0';
  return cString(buf, true);
}

// --- cFileDownloadRequest ------------------------------------------------

cFileDownloadRequest::cFileDownloadRequest(int ID, const char *streamref, 
                                           const char *destdir)
:  cMenuRequest(ID, streamref), title(NULL), bytesDownloaded(0),
   contentLength(-1), destfile(NULL)
{
  this->destdir = strdup(destdir);
}

cFileDownloadRequest::~cFileDownloadRequest() {
  if (destfile) {
    destfile->Close();
    delete destfile;
  }
  if (destdir)
    free(destdir);
  if (title)
    free(title);
}

WebviHandle cFileDownloadRequest::PrepareHandle() {
  if (handle == -1) {
    handle = webvi_new_request(webvi, wvtref, WEBVIREQ_FILE);

    if (handle != -1) {
      webvi_set_opt(webvi, handle, WEBVIOPT_WRITEFUNC, WriteCallback);
      webvi_set_opt(webvi, handle, WEBVIOPT_WRITEDATA, this);
    }
  }

  return handle;
}

ssize_t cFileDownloadRequest::WriteData(const char *ptr, size_t len) {
  if (!destfile) {
    if (!OpenDestFile())
      return -1;
  }

  bytesDownloaded += len;
  return destfile->Write(ptr, len);
}

bool cFileDownloadRequest::OpenDestFile() {
  char *contentType;
  char *url;
  char *ext;
  cString destfilename;
  int fd, i;

  if (handle == -1)
    return false;

  if (destfile)
    delete destfile;

  destfile = new cUnbufferedFile;

  webvi_get_info(webvi, handle, WEBVIINFO_URL, &url);
  webvi_get_info(webvi, handle, WEBVIINFO_STREAM_TITLE, &title);
  webvi_get_info(webvi, handle, WEBVIINFO_CONTENT_TYPE, &contentType);
  webvi_get_info(webvi, handle, WEBVIINFO_CONTENT_LENGTH, &contentLength);

  debug("url: %s", url);
  debug("title: %s", title);
  debug("content type: %s", contentType);
  debug("content length: %ld", contentLength);

  if (!contentType || !url) {
    if(contentType)
      free(contentType);
    if (url)
      free(url);
    return false;
  }

  ext = GetExtension(contentType, url);

  free(url);
  free(contentType);

  char *basename = strdup(title ? title : "???");
  basename = safeFilename(basename);

  i = 1;
  destfilename = cString::sprintf("%s/%s%s", destdir, basename, ext);
  while (true) {
    debug("trying to open %s", (const char *)destfilename);

    fd = destfile->Open(destfilename, O_WRONLY | O_CREAT | O_EXCL, DEFFILEMODE);

    if (fd == -1 && errno == EEXIST)
      destfilename = cString::sprintf("%s/%s-%d%s", destdir, basename, i++, ext);
    else
      break;
  };

  free(basename);
  free(ext);

  if (fd < 0) {
    error("Failed to open file %s: %m", (const char *)destfilename);
    delete destfile;
    destfile = NULL;
    return false;
  }

  info("Saving to %s", (const char *)destfilename);

  return true;
}

char *cFileDownloadRequest::GetExtension(const char *contentType, const char *url) {
  // Get extension from Content-Type
  char *ext = NULL;
  char *ext2 = MimeTypes->ExtensionFromMimeType(contentType);

  // Workaround for buggy servers: If the server claims that the mime
  // type is text/plain, ignore the server and fall back to extracting
  // the extension from the URL. This function should be called only
  // for video, audio or ASX files and therefore text/plain is clearly
  // incorrect.
  if (ext2 && contentType && !strcasecmp(contentType, "text/plain")) {
    debug("Ignoring content type text/plain, getting extension from url.");
    free(ext2);
    ext2 = NULL;
  }

  if (ext2) {
    // Append dot in the start of the extension
    ext = (char *)malloc(strlen(ext2)+2);
    ext[0] = '.';
    ext[1] = '\0';
    strcat(ext, ext2);
    free(ext2);
    return ext;
  }

  // Get extension from URL
  ext = extensionFromUrl(url);
  if (ext)
    return ext;

  // No extension!
  return strdup("");
}

void cFileDownloadRequest::RequestDone(int errorcode, cString pharse) {
  cMenuRequest::RequestDone(errorcode, pharse);
  if (destfile)
    destfile->Close();
}

cDownloadProgress *cFileDownloadRequest::GetProgress(tProgressID downloadid) {
  const char *name;
  if (title)
    name = title;
  else
    name = "???";

  cString dltitle;
  if (IsAborted()) {
    dltitle = cString::sprintf("[%s] %s", tr("Aborted"), name);
  } else {
    dltitle = cString(name);
  }

  return new cDownloadProgress(dltitle, bytesDownloaded, contentLength, 
                               downloadid);
}

// --- cStreamUrlRequest ---------------------------------------------------

WebviHandle cStreamUrlRequest::PrepareHandle() {
  if (handle == -1) {
    handle = webvi_new_request(webvi, wvtref, WEBVIREQ_STREAMURL);

    if (handle != -1) {
      webvi_set_opt(webvi, handle, WEBVIOPT_WRITEFUNC, WriteCallback);
      webvi_set_opt(webvi, handle, WEBVIOPT_WRITEDATA, this);
    }
  }

  return handle;
}

// --- cRequestVector ------------------------------------------------------

cMenuRequest *cRequestVector::FindByHandle(WebviHandle handle) {
  for (int i=0; i<Size(); i++)
    if (At(i)->GetHandle() == handle)
      return At(i);

  return NULL;
}
